@@ -1,39 +1,34 @@
-/*=============================================================================
-
-    This file is part of ARB.
-
-    ARB is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    ARB is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with ARB; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
-
-=============================================================================*/
-/******************************************************************************
-
+/*
     Copyright (C) 2013 Fredrik Johansson
 
-******************************************************************************/
+    This file is part of Arb.
+
+    Arb is free software: you can redistribute it and/or modify it under
+    the terms of the GNU Lesser General Public License (LGPL) as published
+    by the Free Software Foundation; either version 2.1 of the License, or
+    (at your option) any later version.  See <http://www.gnu.org/licenses/>.
+*/
 
 #ifndef FMPZ_EXTRAS_H
 #define FMPZ_EXTRAS_H
 
 #include <limits.h>
-#include "flint.h"
-#include "fmpz.h"
+#include "flint/flint.h"
+#include "flint/fmpz.h"
 
 #ifdef __cplusplus
 extern "C" {
 #endif
 
+#ifndef flint_abort
+#if __FLINT_RELEASE <= 20502
+#define flint_abort abort
+#endif
+#endif
+
+/* currently defined in the arb module, but global to the library */
+double arb_test_multiplier(void);
+
 static __inline__ void
 fmpz_add_inline(fmpz_t z, const fmpz_t x, const fmpz_t y)
 {
@@ -48,6 +43,8 @@ fmpz_add_inline(fmpz_t z, const fmpz_t x, const fmpz_t y)
         fmpz_add(z, x, y);
 }
 
+#if __FLINT_RELEASE < 20600
+
 static __inline__ void
 fmpz_add_si(fmpz_t z, const fmpz_t x, slong y)
 {
@@ -66,6 +63,8 @@ fmpz_sub_si(fmpz_t z, const fmpz_t x, slong y)
         fmpz_add_ui(z, x, -y);
 }
 
+#endif
+
 static __inline__ void
 fmpz_add_si_inline(fmpz_t z, const fmpz_t x, slong y)
 {
@@ -127,35 +126,6 @@ fmpz_add2_fmpz_si_inline(fmpz_t z, const fmpz_t x, const fmpz_t y, slong c)
     fmpz_add_si(z, z, c);
 }
 
-/* sets z = +/- ({src, n} >> shift) where 0 <= shift < FLINT_BITS
-   and the top limb of src is nonzero */
-/* TODO: optimize for result = 1 limb */
-
-static __inline__ void
-fmpz_set_mpn_rshift(fmpz_t z, mp_srcptr src, mp_size_t n, unsigned int shift, int negative)
-{
-    __mpz_struct * zptr;
-    zptr = _fmpz_promote(z);
-
-    if (zptr->_mp_alloc < n)
-        mpz_realloc2(zptr, n * FLINT_BITS);
-
-    if (shift == 0)
-    {
-        flint_mpn_copyi(zptr->_mp_d, src, n);
-    }
-    else
-    {
-        mpn_rshift(zptr->_mp_d, src, n, shift);
-        while (zptr->_mp_d[n - 1] == 0) /* todo: can only do one iter? */
-            n--;
-    }
-
-    zptr->_mp_size = negative ? -n : n;
-    _fmpz_demote_val(z);
-}
-
-/* sets z = +/- {src, n} where n >= 2 and the top limb of src is nonzero */
 static __inline__ void
 fmpz_set_mpn_large(fmpz_t z, mp_srcptr src, mp_size_t n, int negative)
 {
@@ -169,7 +139,6 @@ fmpz_set_mpn_large(fmpz_t z, mp_srcptr src, mp_size_t n, int negative)
     zz->_mp_size = negative ? -n : n;
 }
 
-/* round away from zero */
 static __inline__ void
 fmpz_adiv_q_2exp(fmpz_t z, const fmpz_t x, mp_bitcnt_t exp)
 {
@@ -181,23 +150,11 @@ fmpz_adiv_q_2exp(fmpz_t z, const fmpz_t x, mp_bitcnt_t exp)
         fmpz_fdiv_q_2exp(z, x, exp);
 }
 
-/* sets z = x + y*2^shift */
-static __inline__ void fmpz_add_mul2exp(fmpz_t z, const fmpz_t x, const fmpz_t y, ulong shift)
+static __inline__ void
+_fmpz_set_si_small(fmpz_t x, slong v)
 {
-    fmpz_t t;
-    fmpz_init(t);
-    fmpz_mul_2exp(t, y, shift);
-    fmpz_add(z, x, t);
-    fmpz_clear(t);
-}
-
-static __inline__ void fmpz_sub_mul2exp(fmpz_t z, const fmpz_t x, const fmpz_t y, ulong shift)
-{
-    fmpz_t t;
-    fmpz_init(t);
-    fmpz_mul_2exp(t, y, shift);
-    fmpz_sub(z, x, t);
-    fmpz_clear(t);
+    fmpz_clear(x);
+    *x = v;
 }
 
 slong _fmpz_sub_small_large(const fmpz_t x, const fmpz_t y);
@@ -226,6 +183,20 @@ _fmpz_size(const fmpz_t f)
         return mpz_size(COEFF_TO_PTR(d));
 }
 
+static __inline__ void
+fmpz_ui_mul_ui(fmpz_t r, ulong a, ulong b)
+{
+    if (a < (UWORD(1) << (FLINT_BITS / 2)) && b < (UWORD(1) << (FLINT_BITS / 2)))
+    {
+        fmpz_set_ui(r, a * b);
+    }
+    else
+    {
+        fmpz_set_ui(r, a);
+        fmpz_mul_ui(r, r, b);
+    }
+}
+
 static __inline__ void
 fmpz_ui_pow_ui(fmpz_t x, ulong b, ulong e)
 {
@@ -277,6 +248,15 @@ fmpz_min(fmpz_t z, const fmpz_t x, const fmpz_t y)
 
 void fmpz_lshift_mpn(fmpz_t z, mp_srcptr d, mp_size_t dn, int sgnbit, mp_bitcnt_t shift);
 
+static __inline__ slong
+fmpz_allocated_bytes(const fmpz_t x)
+{
+    if (COEFF_IS_MPZ(*x))
+        return sizeof(__mpz_struct) + COEFF_TO_PTR(*x)->_mp_alloc * sizeof(mp_limb_t);
+    else
+        return 0;
+}
+
 #ifdef __cplusplus
 }
 #endif
